package sun.security.pkcs;

import java.security.MessageDigest;
import java.security.Key;
import java.io.ObjectInputStream;
import java.io.ObjectStreamException;
import java.security.KeyRep;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.security.Provider;
import java.security.Security;
import java.security.spec.InvalidKeySpecException;
import java.security.NoSuchAlgorithmException;
import java.security.KeyFactory;
import java.security.spec.PKCS8EncodedKeySpec;
import sun.security.util.DerOutputStream;
import sun.security.util.Debug;
import java.io.IOException;
import sun.security.util.DerValue;
import java.security.InvalidKeyException;
import java.math.BigInteger;
import sun.security.x509.AlgorithmId;
import java.security.PrivateKey;

public class PKCS8Key implements PrivateKey
{
    private static final long serialVersionUID = -3836890099307167124L;
    protected AlgorithmId algid;
    protected byte[] key;
    protected byte[] encodedKey;
    public static final BigInteger version;
    
    public PKCS8Key() {
    }
    
    private PKCS8Key(AlgorithmId algid, byte[] key) throws InvalidKeyException {
        this.algid = algid;
        this.key = key;
        this.encode();
    }
    
    public static PKCS8Key parse(DerValue derValue) throws IOException {
        PrivateKey key = parseKey(derValue);
        if ((key instanceof PKCS8Key)) {
            return (PKCS8Key)key;
        }
        throw new IOException("Provider did not return PKCS8Key");
    }
    
    public static PrivateKey parseKey(DerValue derValue) throws IOException {
        if (derValue.tag != 48) {
            throw new IOException("corrupt private key");
        }
        BigInteger bigInteger = derValue.data.getBigInteger();
        if (!PKCS8Key.version.equals(bigInteger)) {
            throw new IOException("version mismatch: (supported: " + Debug.toHexString(PKCS8Key.version) + ", parsed: " + Debug.toHexString(bigInteger));
        }
        AlgorithmId parse = AlgorithmId.parse(derValue.data.getDerValue());
        PrivateKey buildPKCS8Key;
        try {
            buildPKCS8Key = buildPKCS8Key(parse, derValue.data.getOctetString());
        }
        catch (InvalidKeyException ex) {
            throw new IOException("corrupt private key");
        }
        if (derValue.data.available() != 0) {
            throw new IOException("excess private key");
        }
        return buildPKCS8Key;
    }
    
    protected void parseKeyBits() throws IOException, InvalidKeyException {
        this.encode();
    }
    
    static PrivateKey buildPKCS8Key(AlgorithmId algorithmId, byte[] array) throws IOException, InvalidKeyException {
        DerOutputStream derOutputStream = new DerOutputStream();
        encode(derOutputStream, algorithmId, array);
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(derOutputStream.toByteArray());
        try {
            return KeyFactory.getInstance(algorithmId.getName()).generatePrivate(keySpec);
        }
        catch (NoSuchAlgorithmException ex) {}
        catch (InvalidKeySpecException ex2) {}
        String property = "";
        try {
            Provider provider = Security.getProvider("SUN");
            if (provider == null) {
                throw new InstantiationException();
            }
            property = provider.getProperty("PrivateKey.PKCS#8." + algorithmId.getName());
            if (property == null) {
                throw new InstantiationException();
            }
            Class<Object> clazz = null;
            try {
                clazz = Class.forName(property);
            }
            catch (ClassNotFoundException ex3) {
                ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
                if (systemClassLoader != null) {
                    clazz = systemClassLoader.loadClass(property);
                }
            }
            Object instance = null;
            if (clazz != null) {
                instance = clazz.newInstance();
            }
            if ((instance instanceof PKCS8Key)) {
                PKCS8Key pkcs8Key = (PKCS8Key)instance;
                pkcs8Key.algid = algorithmId;
                pkcs8Key.key = array;
                pkcs8Key.parseKeyBits();
                return pkcs8Key;
            }
        }
        catch (ClassNotFoundException ex4) {}
        catch (InstantiationException ex5) {}
        catch (IllegalAccessException ex6) {
            throw new IOException(property + " [internal error]");
        }
        PKCS8Key o = new PKCS8Key();
        o.algid = algorithmId;
        o.key = array;
        return o;
    }
    
    @Override
    public String getAlgorithm() {
        return this.algid.getName();
    }
    
    public AlgorithmId getAlgorithmId() {
        return this.algid;
    }
    
    public final void encode(DerOutputStream derOutputStream) throws IOException {
        encode(derOutputStream, this.algid, this.key);
    }
    
    @Override
    public synchronized byte[] getEncoded() {
        byte[] encode = null;
        try {
            encode = this.encode();
        }
        catch (InvalidKeyException ex) {}
        return encode;
    }
    
    @Override
    public String getFormat() {
        return "PKCS#8";
    }
    
    public byte[] encode() throws InvalidKeyException {
        if (this.encodedKey == null) {
            try {
                DerOutputStream derOutputStream = new DerOutputStream();
                this.encode(derOutputStream);
                this.encodedKey = derOutputStream.toByteArray();
            }
            catch (IOException ex) {
                throw new InvalidKeyException("IOException : " + ex.getMessage());
            }
        }
        return (byte[])this.encodedKey.clone();
    }
    
    public void decode(InputStream in) throws InvalidKeyException {
        try {
            DerValue derValue = new DerValue(in);
            if (derValue.tag != 48) {
                throw new InvalidKeyException("invalid key format");
            }
            BigInteger bigInteger = derValue.data.getBigInteger();
            if (!bigInteger.equals(PKCS8Key.version)) {
                throw new IOException("version mismatch: (supported: " + Debug.toHexString(PKCS8Key.version) + ", parsed: " + Debug.toHexString(bigInteger));
            }
            this.algid = AlgorithmId.parse(derValue.data.getDerValue());
            this.key = derValue.data.getOctetString();
            this.parseKeyBits();
            if (derValue.data.available() == 0) {}
        }
        catch (IOException ex) {
            throw new InvalidKeyException("IOException : " + ex.getMessage());
        }
    }
    
    public void decode(byte[] buf) throws InvalidKeyException {
        this.decode(new ByteArrayInputStream(buf));
    }
    
    protected Object writeReplace() throws ObjectStreamException {
        return new KeyRep(KeyRep.Type.PRIVATE, this.getAlgorithm(), this.getFormat(), this.getEncoded());
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException {
        try {
            this.decode(objectInputStream);
        }
        catch (InvalidKeyException ex) {
            ex.printStackTrace();
            throw new IOException("deserialized key is invalid: " + ex.getMessage());
        }
    }
    
    static void encode(DerOutputStream derOutputStream, AlgorithmId algorithmId, byte[] array) throws IOException {
        DerOutputStream out = new DerOutputStream();
        out.putInteger(PKCS8Key.version);
        algorithmId.encode(out);
        out.putOctetString(array);
        derOutputStream.write((byte)48, out);
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof Key)) {
            byte[] digesta;
            if (this.encodedKey != null) {
                digesta = this.encodedKey;
            }
            else {
                digesta = this.getEncoded();
            }
            return MessageDigest.isEqual(digesta, ((Key)o).getEncoded());
        }
        else {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        byte[] encoded = this.getEncoded();
        for (int i = 1; i < encoded.length; ++i) {
            n += encoded[i] * i;
        }
        return n;
    }
    
    static {
        version = BigInteger.ZERO;
    }
}
